import os
import numpy as np
import netCDF4
import contextily as ctx
import matplotlib.pyplot as plt
from windrose import WindroseAxes
import matplotlib.animation as animation

main_path = r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\00-PhD\P3_aeolis\ERA5 data'

# Input Morocco
# ERA5_nc = os.path.join(main_path, 'adaptor.mars.internal-1699882598.4786408-21772-7-4a8d968b-1760-4edf-a225-6819e84e90b2.nc')
# gif_name = 'barchan_maroc_wind.gif'
# wind_txt_name = 'wind_morocco.txt'

# Input NPZK 2012-2017
# ERA5_nc = os.path.join(main_path, 'NPZK', 'adaptor.mars.internal-1700792980.06934-14218-18-dfb96072-2dae-48c8-be12-6172c8d4a88e.nc')
# gif_name = 'NPZK_2012_2017.gif'
# wind_txt_name = 'wind_NPZK_2012_2017.txt'

# Input NPZK 2018-2023
# ERA5_nc = os.path.join(main_path, 'NPZK', 'adaptor.mars.internal-1700786704.1406364-27062-12-bedf5c83-077e-4148-ab4d-40437e0ff86c.nc')
# gif_name = 'NPZK_2018_2023.gif'
# wind_txt_name = 'wind_NPZK_2018_2023.txt'

# Input Hors 2012-2017
# ERA5_nc = os.path.join(main_path, 'adaptor.mars.internal-1702380851.8828418-29107-14-f927933e-ff75-4fd2-9f02-f174f694bcaf.nc')
# gif_name = 'Hors_2012_2017.gif'
# wind_txt_name = 'wind_Hors_2012_2017.txt'

# Input Brazil
ERA5_nc = os.path.join(main_path, 'adaptor.mars.internal-1702489298.1591487-8385-5-6042e39e-b906-4ed7-93a3-ad156b8e3c1a.nc')
gif_name = 'Brazil_2012_2017.gif'
wind_txt_name = 'wind_Brazil_2012_2017.txt'

source_ctx = ctx.providers.Esri.WorldGrayCanvas

ilat = 2
ilon = 2
it_max = 500

# Read data
with netCDF4.Dataset(ERA5_nc, 'r') as ds:
    print(ds)
    
    time = ds.variables['time'][:]
    print(time[-1])

    lon = ds.variables['longitude'][:]
    lat = ds.variables['latitude'][:]

    u10 = ds.variables['u10'][:, :, :]
    v10 = ds.variables['v10'][:, :, :] 

lat_box = [np.min(lat), np.min(lat), np.max(lat), np.max(lat), np.min(lat)]
lon_box = [np.min(lon), np.max(lon), np.max(lon), np.min(lon), np.min(lon)]

if len(np.shape(u10)) == 4:
    u10 = u10[:, 0, :, :]
    v10 = v10[:, 0, :, :]


# Convert to magnitude and direction
ws = np.sqrt(u10**2 + v10**2)
# wd_rad = np.arctan2(u10, v10)           # compute radians
# wd_cart = np.rad2deg(wd_rad)                 # convert to degrees
# wd_naut = 270 - wd_cart                       # convert to nautical
# wd_naut = (wd_naut + 360) % 360                # correction

wd_rad1 = np.arctan2(u10, v10)    
wd_rad = np.arctan2(v10, u10)           # Calculate the angle in radians
wd_cart = np.rad2deg(wd_rad)            # Convert radians to degrees
wd_naut = (270 - wd_cart) % 360         # Adjust for nautical direction

w_time = (time - time[0]) * 3600

np.savetxt(os.path.join(main_path, wind_txt_name), np.array([w_time, ws[:, ilat, ilon], wd_naut[:, ilat, ilon]]).T)

# Start plotting
fig, axs = plt.subplot_mosaic([['a)', 'b)', 'c)'],['d)', 'd)', 'd)']])
fig.set_size_inches(18., 9.)

# Global map
ax = axs['a)']
ax.plot(lon_box, lat_box, 'r')
ax.plot( lon[ilon], lat[ilat], 'r*')
ax.set_ylim([lat[ilat]-5, lat[ilat]+5])
ax.set_xlim([lon[ilon]-5, lon[ilon]+5])
ctx.add_basemap(ax, source=source_ctx, crs='EPSG:4326')#, attribution_size=5)
ax.set_aspect('equal')

# Spatial variation
ax = axs['b)']
pc = ax.pcolormesh(lon, lat, ws[0, :, :], vmin=0, vmax=10)
ax.quiver(lon, lat, np.average(u10[:, :, :], axis=0), np.average(v10[:, :, :], axis=0), alpha=0.5)
quiv = ax.quiver(lon, lat, u10[0, :, :], v10[0, :, :], color='r', alpha=0.5)
ax.plot( lon[ilon], lat[ilat], 'r*')
fig.colorbar(pc, ax=ax)
ax.set_aspect('equal')

# Timeseries
ax = axs['d)']
ax.plot(time[:]/(24*365), ws[:, ilat, ilon], 'k')
time_point, = ax.plot(time[0]/(24*365), ws[0, ilat, ilon], 'r*')
ax.set_xlim([np.min(time/(24*365)), np.max(time[it_max]/(24*365))])
ax.set_ylabel('Windspeed [m/s]')
ax.set_xlabel('Time [years]')

# Windrose
ax = axs['c)']
ax.set_axis_off()

rect=[0.6,0.5,0.4,0.4] 
ax = WindroseAxes(fig, rect)
fig.add_axes(ax)
ax.bar(wd_naut[:, ilat, ilon], ws[:, ilat, ilon], normed=True, opening=0.8, bins=range(0,15,2), edgecolor='white', nsector=36)
# windrose_point, = ax.plot(wd_rad1[0, ilat, ilon], ws[0, ilat, ilon], 'r*') # Convert back to cartisian radians (?)

ax.set_legend()
# plt.show()

def animate(it):
    print(it)

    # update wind speed
    pc.set_array(ws[it,:,:])

    # update quiver
    quiv.set_UVC(u10[it, :, :], v10[it, :, :])

    # update point in timeseries
    time_point.set_data(time[it]/(24*365), ws[it, ilat, ilon]) 
    
    # update point in windrose
    # windrose_point.set_data(wd_rad1[it, ilat, ilon], ws[it, ilat, ilon]) 

    return 

ani = animation.FuncAnimation(fig, animate, repeat=False, frames=it_max)# , interval=50)

# To save the animation using Pillow as a gif
writer = animation.PillowWriter(fps=20, bitrate=1800)
ani.save(os.path.join(main_path, gif_name), writer=writer)


