import numpy as np
import numpy as np
import glob, os
import matplotlib.pyplot as plt
import timeit
from matplotlib.colors import ListedColormap
from shapely.geometry import Polygon, Point
from scipy.interpolate import griddata
from scipy import ndimage

main_folder = r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\00-PhD\P3_aeolis\NPZK'


# Load x,y,z,masks
x = np.load(os.path.join(main_folder, '02_data_npy', 'x_NPZK.npy'))#[2:-2, 2:-2]
y = np.load(os.path.join(main_folder, '02_data_npy', 'y_NPZK.npy'))#[2:-2, 2:-2]
z = np.load(os.path.join(main_folder, '02_data_npy', 'z_NPZK_2013.npy'))#[2:-2, 2:-2]

mask_notches = np.load(os.path.join(main_folder, '02_data_npy', 'mask_1_notches_NPZK.npy'))#[2:-2, 2:-2]
mask_foredune = np.load(os.path.join(main_folder, '02_data_npy', 'mask_2_foredune_NPZK.npy'))#[2:-2, 2:-2]
mask_foredune_b = np.load(os.path.join(main_folder, '02_data_npy', 'mask_2b_foredune_b_NPZK.npy'))#[2:-2, 2:-2]
mask_backdune = np.load(os.path.join(main_folder, '02_data_npy', 'mask_3b_backdune_b_NPZK.npy'))#[2:-2, 2:-2]


# Fill nans / 0
i_nan = (z == 0)
x_array = x[~i_nan].flatten()
y_array = y[~i_nan].flatten()
z_array = z[~i_nan].flatten()
z = griddata(np.vstack((x_array, y_array)).T, z_array, (x, y), method='nearest')

# Smoothening 2D z
z = ndimage.uniform_filter(z, size=5, mode='constant')

# Write x,y
np.savetxt(os.path.join(main_folder, 'x_NPZK.grd'), x[2:-2, 2:-2]) 
np.savetxt(os.path.join(main_folder, 'y_NPZK.grd'), y[2:-2, 2:-2])

# Write zb
np.savetxt(os.path.join(main_folder, 'z_NPZK.grd'), z[2:-2, 2:-2])

# Write zne
zne = z.copy() - 0.5
zne[(zne > 6.)] = 5.5
np.savetxt(os.path.join(main_folder, 'zne_NPZK.grd'), zne[2:-2, 2:-2])

# Write zroot
zroot = z.copy()
zroot[:] = -10
# zroot[mask_foredune] = z[mask_foredune] 
zroot[mask_backdune] = z[mask_backdune] 
np.savetxt(os.path.join(main_folder, 'zroot_NPZK.grd'), zroot[2:-2, 2:-2])

# Write vegetation density
veg = np.zeros(np.shape(z))
veg[mask_foredune * (z > 10)] = 0.2
veg[mask_backdune] = 1.0
np.savetxt(os.path.join(main_folder, 'veg_NPZK.grd'), veg[2:-2, 2:-2])

# Write vegetation growth mask
vver_mask = np.zeros(np.shape(z))
vver_mask[mask_foredune_b] = 1.0
np.savetxt(os.path.join(main_folder, 'vver_mask_NPZK.grd'), vver_mask[2:-2, 2:-2])

# Write threshold mask
multi_mask = np.zeros(np.shape(mask_foredune)) + 1.
# multi_mask[mask_foredune_b] += 10.
add_mask = multi_mask.copy() * 0. 
add_mask[mask_foredune_b * (z > 8)] = 10.
threshold_mask = multi_mask + add_mask * 1.0j
np.savetxt(os.path.join(main_folder, 'threshold_mask_NPZK.grd'), threshold_mask[2:-2, 2:-2])

# Write tide and wave masks
multi_mask = np.zeros(np.shape(mask_foredune)) + 1
add_mask = multi_mask * 0

# Filter out nan values
multi_mask[mask_backdune] = 0.
add_mask[mask_backdune] = -10

# Create complex masks
tide_mask = multi_mask + add_mask * 1.0j
wave_mask = multi_mask

np.savetxt(os.path.join(main_folder, 'tide_mask_NPZK.grd'), tide_mask[2:-2, 2:-2])
np.savetxt(os.path.join(main_folder, 'wave_mask_NPZK.grd'), wave_mask[2:-2, 2:-2])

# Plotting

fig,axs = plt.subplots(2,4)

ax=axs[0,0]
pc = ax.pcolormesh(x, y, z)
ax.set_aspect('equal')
ax.set_title('Z')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[0,1]
pc = ax.pcolormesh(x, y, zne)
ax.set_aspect('equal')
ax.set_title('Zne')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[0,2]
pc = ax.pcolormesh(x, y, veg)
ax.set_aspect('equal')
ax.set_title('veg')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[0,3]
pc = ax.pcolormesh(x, y, vver_mask)
ax.set_aspect('equal')
ax.set_title('vver')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[1,0]
pc = ax.pcolormesh(x, y, np.real(tide_mask))
ax.set_aspect('equal')
ax.set_title('tide real')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[1,1]
pc = ax.pcolormesh(x, y, np.real(threshold_mask)) # wave_mask
ax.set_aspect('equal')
ax.set_title('Threshold real')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[1,2]
pc = ax.pcolormesh(x, y, np.imag(tide_mask))
ax.set_aspect('equal')
ax.set_title('tide imag')
fig.colorbar(pc, ax=ax, location='bottom')

ax=axs[1,3]
pc = ax.pcolormesh(x, y, np.imag(threshold_mask)) # wave_mask
ax.set_aspect('equal')
ax.set_title('Threshold imag')
fig.colorbar(pc, ax=ax, location='bottom')

plt.show()
