from scipy.interpolate import griddata
from shapely.geometry import Point, Polygon
import glob, os
import numpy as np
import matplotlib.pyplot as plt
import scipy
# import alphashape
from matplotlib.colors import ListedColormap
import matplotlib as mpl
import pandas as pd


# Define Sand Engine colormap
cmap_ZM = ListedColormap(np.load(os.path.join(r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\01-Projects\Zandmotor', '04_scripts', 'SE_cmap.npy')))

# Files
xyz_folder = os.path.join(r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\01-Projects\Zandmotor', '01_data_raw', 'LiDAR', 'xyz')

main_folder = r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\00-PhD\P3_aeolis\NPZK'
output_loc = os.path.join(main_folder, '00_data_npy')

if not os.path.exists(output_loc):
    os.makedirs(output_loc)

# Read lidar info
lidar_info = np.genfromtxt(os.path.join(r'c:\Users\weste_bt\OneDrive - Stichting Deltares\Documents\00-PhD\P3_aeolis\NPZK\xyz_NPZK.txt'), dtype='str')
lidar_files = lidar_info[:,0]
timestamps_str = lidar_info[:,3]

# Convert times
t_datetimes = [pd.to_datetime(timestamp_str, format='%d_%m_%Y').to_datetime64() for timestamp_str in timestamps_str]
        
# Create meshgrid (coordinates from Ruessink)
x0 = 98000 + 400
xn = 98000 + 900
y0 = 493000 + 100
yn = 493000 + 900

dx = 5. # [m] Gridcell size for meshgrid # (first dx=2)
x_array = np.arange(x0, xn, dx)
y_array = np.arange(y0, yn, dx)
x_grid, y_grid = np.meshgrid(x_array, y_array) 


# fig, axs = plt.subplots(len(lidar_files),2)

for it_lidar, file in enumerate(lidar_files):

    #%% Read JETSKI and filter
    xyz_lidar = np.loadtxt(os.path.join(xyz_folder, 'LiDAR_' + file + '.xyz'))
    mask = (xyz_lidar[:, 2] != np.NAN)
    x_lidar = xyz_lidar[mask, 0]
    y_lidar = xyz_lidar[mask, 1]
    z_lidar = xyz_lidar[mask, 2]
    print('Data loaded')

    fig,ax=plt.subplots()
    # ax.scatter(x_lidar,y_lidar,c=z_lidar,cmap=cmap_ZM)
    # ax.set_aspect('equal')
    # plt.show()

    # Interpolate onto grid
    z_grid = griddata(np.vstack((x_lidar, y_lidar)).T, z_lidar, (x_grid, y_grid), method='nearest')
    print('Lidar interpolated')

    if it_lidar == 0:
        z0_grid = z_grid.copy()
        np.save(os.path.join(output_loc, 'x_lidar_ZM_dx5.npy'), x_grid)
        np.save(os.path.join(output_loc, 'y_lidar_ZM_dx5.npy'), y_grid)

    dz_grid = z_grid - z0_grid

    # Store ZM
    np.save(os.path.join(output_loc, 'z_lidar_ZM_' + timestamps_str[it_lidar] + '_dx5.npy'), z_grid)

    #%% Plotting
    # axs[it_lidar, 0].pcolormesh(x_grid, y_grid, z_grid, vmin=-12, vmax=6, cmap=cmap_ZM)
    # axs[it_lidar, 1].pcolormesh(x_grid, y_grid, dz_grid, vmin=-5, vmax=5, cmap='RdBu')


# for ax_row in axs:
#     for ax in ax_row:
#         ax.set_aspect('equal')

# fig.set_size_inches(30., 8.)
# plt.tight_layout()
# fig.savefig(os.path.join(output_loc, 'bed_levels_NPZK.png'), dpi=300)
# plt.close('all')